@@ -65,6 +65,7 @@
 #define HAVE_SYS_UIO_H 1
 #define HAVE_TIME_H 1
 #define HAVE_UTIME_H 1
+#define HAVE_STDATOMIC_H 1
 #define HAVE_X86INTRIN_H 1
 #if defined(__x86_64__)
 #define HAVE_X86INTRIN_H 1
@@ -266,6 +267,7 @@
 #define HAVE_ARC4RANDOM_BUF 1
 #define HAVE_ATAN2L 1
 #define HAVE_ATAN2F 1
+#define HAVE_DECL_ATOMIC_SIGNAL_FENCE 1
 #define HAVE_CHMOD 1
 #define HAVE_CHOWN 1
 #define HAVE_CHROOT 1
