@@ -1,7 +1,8 @@
+# frozen_string_literal: false
 #
 #   irb/extend-command.rb - irb extend command
 #   	$Release Version: 0.9.6$
-#   	$Revision: 47114 $
+#   	$Revision$
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -31,7 +32,7 @@ module IRB # :nodoc:
 
     # Displays current configuration.
     #
-    # Modifing the configuration is achieved by sending a message to IRB.conf.
+    # Modifying the configuration is achieved by sending a message to IRB.conf.
     def irb_context
       IRB.CurrentContext
     end
@@ -44,77 +45,200 @@ module IRB # :nodoc:
       [:quit, :irb_exit, OVERRIDE_PRIVATE_ONLY],
     ]
 
+
     @EXTEND_COMMANDS = [
-      [:irb_current_working_workspace, :CurrentWorkingWorkspace, "irb/cmd/chws",
-       [:irb_print_working_workspace, OVERRIDE_ALL],
-       [:irb_cwws, OVERRIDE_ALL],
-       [:irb_pwws, OVERRIDE_ALL],
-       [:cwws, NO_OVERRIDE],
-       [:pwws, NO_OVERRIDE],
-       [:irb_current_working_binding, OVERRIDE_ALL],
-       [:irb_print_working_binding, OVERRIDE_ALL],
-       [:irb_cwb, OVERRIDE_ALL],
-       [:irb_pwb, OVERRIDE_ALL],
-    ],
-    [:irb_change_workspace, :ChangeWorkspace, "irb/cmd/chws",
-     [:irb_chws, OVERRIDE_ALL],
-     [:irb_cws, OVERRIDE_ALL],
-     [:chws, NO_OVERRIDE],
-     [:cws, NO_OVERRIDE],
-     [:irb_change_binding, OVERRIDE_ALL],
-     [:irb_cb, OVERRIDE_ALL],
-     [:cb, NO_OVERRIDE]],
+      [
+        :irb_current_working_workspace, :CurrentWorkingWorkspace, "cmd/chws",
+        [:cwws, NO_OVERRIDE],
+        [:pwws, NO_OVERRIDE],
+        [:irb_print_working_workspace, OVERRIDE_ALL],
+        [:irb_cwws, OVERRIDE_ALL],
+        [:irb_pwws, OVERRIDE_ALL],
+        [:irb_current_working_binding, OVERRIDE_ALL],
+        [:irb_print_working_binding, OVERRIDE_ALL],
+        [:irb_cwb, OVERRIDE_ALL],
+        [:irb_pwb, OVERRIDE_ALL],
+      ],
+      [
+        :irb_change_workspace, :ChangeWorkspace, "cmd/chws",
+        [:chws, NO_OVERRIDE],
+        [:cws, NO_OVERRIDE],
+        [:irb_chws, OVERRIDE_ALL],
+        [:irb_cws, OVERRIDE_ALL],
+        [:irb_change_binding, OVERRIDE_ALL],
+        [:irb_cb, OVERRIDE_ALL],
+        [:cb, NO_OVERRIDE],
+      ],
 
-    [:irb_workspaces, :Workspaces, "irb/cmd/pushws",
-     [:workspaces, NO_OVERRIDE],
-     [:irb_bindings, OVERRIDE_ALL],
-     [:bindings, NO_OVERRIDE]],
-    [:irb_push_workspace, :PushWorkspace, "irb/cmd/pushws",
-     [:irb_pushws, OVERRIDE_ALL],
-     [:pushws, NO_OVERRIDE],
-     [:irb_push_binding, OVERRIDE_ALL],
-     [:irb_pushb, OVERRIDE_ALL],
-     [:pushb, NO_OVERRIDE]],
-    [:irb_pop_workspace, :PopWorkspace, "irb/cmd/pushws",
-     [:irb_popws, OVERRIDE_ALL],
-     [:popws, NO_OVERRIDE],
-     [:irb_pop_binding, OVERRIDE_ALL],
-     [:irb_popb, OVERRIDE_ALL],
-     [:popb, NO_OVERRIDE]],
+      [
+        :irb_workspaces, :Workspaces, "cmd/pushws",
+        [:workspaces, NO_OVERRIDE],
+        [:irb_bindings, OVERRIDE_ALL],
+        [:bindings, NO_OVERRIDE],
+      ],
+      [
+        :irb_push_workspace, :PushWorkspace, "cmd/pushws",
+        [:pushws, NO_OVERRIDE],
+        [:irb_pushws, OVERRIDE_ALL],
+        [:irb_push_binding, OVERRIDE_ALL],
+        [:irb_pushb, OVERRIDE_ALL],
+        [:pushb, NO_OVERRIDE],
+      ],
+      [
+        :irb_pop_workspace, :PopWorkspace, "cmd/pushws",
+        [:popws, NO_OVERRIDE],
+        [:irb_popws, OVERRIDE_ALL],
+        [:irb_pop_binding, OVERRIDE_ALL],
+        [:irb_popb, OVERRIDE_ALL],
+        [:popb, NO_OVERRIDE],
+      ],
 
-    [:irb_load, :Load, "irb/cmd/load"],
-    [:irb_require, :Require, "irb/cmd/load"],
-    [:irb_source, :Source, "irb/cmd/load",
-     [:source, NO_OVERRIDE]],
+      [
+        :irb_load, :Load, "cmd/load"],
+      [
+        :irb_require, :Require, "cmd/load"],
+      [
+        :irb_source, :Source, "cmd/load",
+        [:source, NO_OVERRIDE],
+      ],
 
-    [:irb, :IrbCommand, "irb/cmd/subirb"],
-    [:irb_jobs, :Jobs, "irb/cmd/subirb",
-     [:jobs, NO_OVERRIDE]],
-    [:irb_fg, :Foreground, "irb/cmd/subirb",
-     [:fg, NO_OVERRIDE]],
-    [:irb_kill, :Kill, "irb/cmd/subirb",
-     [:kill, OVERRIDE_PRIVATE_ONLY]],
+      [
+        :irb, :IrbCommand, "cmd/subirb"],
+      [
+        :irb_jobs, :Jobs, "cmd/subirb",
+        [:jobs, NO_OVERRIDE],
+      ],
+      [
+        :irb_fg, :Foreground, "cmd/subirb",
+        [:fg, NO_OVERRIDE],
+      ],
+      [
+        :irb_kill, :Kill, "cmd/subirb",
+        [:kill, OVERRIDE_PRIVATE_ONLY],
+      ],
 
-    [:irb_help, :Help, "irb/cmd/help",
-     [:help, NO_OVERRIDE]],
+      [
+        :irb_debug, :Debug, "cmd/debug",
+        [:debug, NO_OVERRIDE],
+      ],
+      [
+        :irb_edit, :Edit, "cmd/edit",
+        [:edit, NO_OVERRIDE],
+      ],
+      [
+        :irb_break, :Break, "cmd/break",
+      ],
+      [
+        :irb_catch, :Catch, "cmd/catch",
+      ],
+      [
+        :irb_next, :Next, "cmd/next"
+      ],
+      [
+        :irb_delete, :Delete, "cmd/delete",
+        [:delete, NO_OVERRIDE],
+      ],
+      [
+        :irb_step, :Step, "cmd/step",
+        [:step, NO_OVERRIDE],
+      ],
+      [
+        :irb_continue, :Continue, "cmd/continue",
+        [:continue, NO_OVERRIDE],
+      ],
+      [
+        :irb_finish, :Finish, "cmd/finish",
+        [:finish, NO_OVERRIDE],
+      ],
+      [
+        :irb_backtrace, :Backtrace, "cmd/backtrace",
+        [:backtrace, NO_OVERRIDE],
+        [:bt, NO_OVERRIDE],
+      ],
+      [
+        :irb_debug_info, :Info, "cmd/info",
+        [:info, NO_OVERRIDE],
+      ],
 
+      [
+        :irb_help, :Help, "cmd/help",
+        [:show_doc, NO_OVERRIDE],
+        [:help, NO_OVERRIDE],
+      ],
+
+      [
+        :irb_info, :IrbInfo, "cmd/irb_info"
+      ],
+
+      [
+        :irb_ls, :Ls, "cmd/ls",
+        [:ls, NO_OVERRIDE],
+      ],
+
+      [
+        :irb_measure, :Measure, "cmd/measure",
+        [:measure, NO_OVERRIDE],
+      ],
+
+      [
+        :irb_show_source, :ShowSource, "cmd/show_source",
+        [:show_source, NO_OVERRIDE],
+      ],
+
+      [
+        :irb_whereami, :Whereami, "cmd/whereami",
+        [:whereami, NO_OVERRIDE],
+      ],
+      [
+        :irb_show_cmds, :ShowCmds, "cmd/show_cmds",
+        [:show_cmds, NO_OVERRIDE],
+      ]
     ]
 
-    # Installs the default irb commands:
-    #
-    # +irb_current_working_workspace+::   Context#main
-    # +irb_change_workspace+::            Context#change_workspace
-    # +irb_workspaces+::                  Context#workspaces
-    # +irb_push_workspace+::              Context#push_workspace
-    # +irb_pop_workspace+::               Context#pop_workspace
-    # +irb_load+::                        #irb_load
-    # +irb_require+::                     #irb_require
-    # +irb_source+::                      IrbLoader#source_file
-    # +irb+::                             IRB.irb
-    # +irb_jobs+::                        JobManager
-    # +irb_fg+::                          JobManager#switch
-    # +irb_kill+::                        JobManager#kill
-    # +irb_help+::                        IRB@Command+line+options
+
+    @@commands = []
+
+    def self.all_commands_info
+      return @@commands unless @@commands.empty?
+      user_aliases = IRB.CurrentContext.command_aliases.each_with_object({}) do |(alias_name, target), result|
+        result[target] ||= []
+        result[target] << alias_name
+      end
+
+      @EXTEND_COMMANDS.each do |cmd_name, cmd_class, load_file, *aliases|
+        if !defined?(ExtendCommand) || !ExtendCommand.const_defined?(cmd_class, false)
+          require_relative load_file
+        end
+
+        klass = ExtendCommand.const_get(cmd_class, false)
+        aliases = aliases.map { |a| a.first }
+
+        if additional_aliases = user_aliases[cmd_name]
+          aliases += additional_aliases
+        end
+
+        display_name = aliases.shift || cmd_name
+        @@commands << { display_name: display_name, description: klass.description, category: klass.category }
+      end
+
+      @@commands
+    end
+
+    # Convert a command name to its implementation class if such command exists
+    def self.load_command(command)
+      command = command.to_sym
+      @EXTEND_COMMANDS.each do |cmd_name, cmd_class, load_file, *aliases|
+        next if cmd_name != command && aliases.all? { |alias_name, _| alias_name != command }
+
+        if !defined?(ExtendCommand) || !ExtendCommand.const_defined?(cmd_class, false)
+          require_relative load_file
+        end
+        return ExtendCommand.const_get(cmd_class, false)
+      end
+      nil
+    end
+
+    # Installs the default irb commands.
     def self.install_extend_commands
       for args in @EXTEND_COMMANDS
         def_extend_command(*args)
@@ -137,20 +261,24 @@ module IRB # :nodoc:
       end
 
       if load_file
+        kwargs = ", **kwargs" if RUBY_ENGINE == "ruby" && RUBY_VERSION >= "2.7.0"
         line = __LINE__; eval %[
-          def #{cmd_name}(*opts, &b)
-            require "#{load_file}"
+          def #{cmd_name}(*opts#{kwargs}, &b)
+            require_relative "#{load_file}"
             arity = ExtendCommand::#{cmd_class}.instance_method(:execute).arity
             args = (1..(arity < 0 ? ~arity : arity)).map {|i| "arg" + i.to_s }
-            args << "*opts" if arity < 0
+            args << "*opts#{kwargs}" if arity < 0
             args << "&block"
             args = args.join(", ")
             line = __LINE__; eval %[
-              def #{cmd_name}(\#{args})
-            ExtendCommand::#{cmd_class}.execute(irb_context, \#{args})
+              unless singleton_class.class_variable_defined?(:@@#{cmd_name}_)
+                singleton_class.class_variable_set(:@@#{cmd_name}_, true)
+                def self.#{cmd_name}_(\#{args})
+                  ExtendCommand::#{cmd_class}.execute(irb_context, \#{args})
+                end
               end
             ], nil, __FILE__, line
-            send :#{cmd_name}, *opts, &b
+            __send__ :#{cmd_name}_, *opts#{kwargs}, &b
           end
         ], nil, __FILE__, line
       else
@@ -184,7 +312,7 @@ module IRB # :nodoc:
           alias_method to, from
         }
       else
-        print "irb: warn: can't alias #{to} from #{from}.\n"
+        Kernel.print "irb: warn: can't alias #{to} from #{from}.\n"
       end
     end
 
@@ -211,18 +339,16 @@ module IRB # :nodoc:
     CE = ContextExtender # :nodoc:
 
     @EXTEND_COMMANDS = [
-      [:eval_history=, "irb/ext/history.rb"],
-      [:use_tracer=, "irb/ext/tracer.rb"],
-      [:math_mode=, "irb/ext/math-mode.rb"],
-      [:use_loader=, "irb/ext/use-loader.rb"],
-      [:save_history=, "irb/ext/save-history.rb"],
+      [:eval_history=, "ext/history.rb"],
+      [:use_tracer=, "ext/tracer.rb"],
+      [:use_loader=, "ext/use-loader.rb"],
+      [:save_history=, "ext/save-history.rb"],
     ]
 
     # Installs the default context extensions as irb commands:
     #
     # Context#eval_history=::   +irb/ext/history.rb+
     # Context#use_tracer=::     +irb/ext/tracer.rb+
-    # Context#math_mode=::      +irb/ext/math-mode.rb+
     # Context#use_loader=::     +irb/ext/use-loader.rb+
     # Context#save_history=::   +irb/ext/save-history.rb+
     def self.install_extend_commands
@@ -239,8 +365,8 @@ module IRB # :nodoc:
       line = __LINE__; Context.module_eval %[
         def #{cmd_name}(*opts, &b)
           Context.module_eval {remove_method(:#{cmd_name})}
-          require "#{load_file}"
-          send :#{cmd_name}, *opts, &b
+          require_relative "#{load_file}"
+          __send__ :#{cmd_name}, *opts, &b
         end
         for ali in aliases
           alias_method ali, cmd_name
@@ -263,8 +389,8 @@ module IRB # :nodoc:
       module_eval %[
         alias_method alias_name, base_method
         def #{base_method}(*opts)
-          send :#{extend_method}, *opts
-          send :#{alias_name}, *opts
+          __send__ :#{extend_method}, *opts
+          __send__ :#{alias_name}, *opts
         end
       ]
     end
@@ -279,8 +405,8 @@ module IRB # :nodoc:
       module_eval %[
         alias_method alias_name, base_method
         def #{base_method}(*opts)
-          send :#{alias_name}, *opts
-          send :#{extend_method}, *opts
+          __send__ :#{alias_name}, *opts
+          __send__ :#{extend_method}, *opts
         end
       ]
     end
@@ -305,4 +431,3 @@ module IRB # :nodoc:
     end
   end
 end
-
