@@ -1,15 +1,19 @@
 # frozen_string_literal: true
 
-require 'rubygems/command'
-require 'rubygems/commands/query_command'
+require_relative "../command"
+require_relative "../query_utils"
 
-class Gem::Commands::InfoCommand < Gem::Commands::QueryCommand
+class Gem::Commands::InfoCommand < Gem::Command
+  include Gem::QueryUtils
 
   def initialize
-    super "info", "Show information for the given gem"
+    super "info", "Show information for the given gem",
+         :name => //, :domain => :local, :details => false, :versions => true,
+         :installed => nil, :version => Gem::Requirement.default
 
-    remove_option('--name-matches')
-    remove_option('-d')
+    add_query_options
+
+    remove_option("-d")
 
     defaults[:details] = true
     defaults[:exact] = true
@@ -31,5 +35,4 @@ class Gem::Commands::InfoCommand < Gem::Commands::QueryCommand
   def defaults_str
     "--local"
   end
-
 end
