@@ -18,7 +18,7 @@ module DEBUGGER__
   module SkipPathHelper
     def skip_path?(path)
       !path ||
-      CONFIG.skip? ||
+      DEBUGGER__.skip? ||
       ThreadClient.current.management? ||
       skip_internal_path?(path) ||
       skip_config_skip_path?(path)
@@ -29,7 +29,7 @@ module DEBUGGER__
     end
 
     def skip_internal_path?(path)
-      path.start_with?(__dir__) || path.start_with?('<internal:')
+      path.start_with?(__dir__) || path.delete_prefix('!eval:').start_with?('<internal:')
     end
 
     def skip_location?(loc)
@@ -38,6 +38,13 @@ module DEBUGGER__
     end
   end
 
+  module GlobalVariablesHelper
+    SKIP_GLOBAL_LIST = %i[$= $KCODE $-K $SAFE].freeze
+    def safe_global_variables
+      global_variables.reject{|name| SKIP_GLOBAL_LIST.include? name }
+    end
+  end
+
   class ThreadClient
     def self.current
       if thc = Thread.current[:DEBUGGER__ThreadClient]
@@ -50,6 +57,7 @@ module DEBUGGER__
 
     include Color
     include SkipPathHelper
+    include GlobalVariablesHelper
 
     attr_reader :thread, :id, :recorder, :check_bp_fulfillment_map
 
@@ -291,8 +299,10 @@ module DEBUGGER__
       end
 
       if event != :pause
-        show_src
-        show_frames CONFIG[:show_frames]
+        unless bp&.skip_src
+          show_src
+          show_frames CONFIG[:show_frames]
+        end
 
         set_mode :waiting
 
@@ -328,11 +338,15 @@ module DEBUGGER__
       @step_tp.disable if @step_tp
 
       thread = Thread.current
+      subsession_id = SESSION.subsession_id
 
       if SUPPORT_TARGET_THREAD
         @step_tp = TracePoint.new(*events){|tp|
-          next if SESSION.break_at? tp.path, tp.lineno
-          next if !yield(tp.event)
+          if SESSION.stop_stepping? tp.path, tp.lineno, subsession_id
+            tp.disable
+            next
+          end
+          next if !yield(tp)
           next if tp.path.start_with?(__dir__)
           next if tp.path.start_with?('<internal:trace_point>')
           next unless File.exist?(tp.path) if CONFIG[:skip_nosrc]
@@ -347,8 +361,11 @@ module DEBUGGER__
       else
         @step_tp = TracePoint.new(*events){|tp|
           next if thread != Thread.current
-          next if SESSION.break_at? tp.path, tp.lineno
-          next if !yield(tp.event)
+          if SESSION.stop_stepping? tp.path, tp.lineno, subsession_id
+            tp.disable
+            next
+          end
+          next if !yield(tp)
           next if tp.path.start_with?(__dir__)
           next if tp.path.start_with?('<internal:trace_point>')
           next unless File.exist?(tp.path) if CONFIG[:skip_nosrc]
@@ -384,15 +401,19 @@ module DEBUGGER__
       end
     end
 
-    def frame_eval_core src, b
+    def frame_eval_core src, b, binding_location: false
       saved_target_frames = @target_frames
       saved_current_frame_index = @current_frame_index
 
       if b
-        f, _l = b.source_location
+        file, lineno = b.source_location
 
         tp_allow_reentry do
-          b.eval(src, "(rdbg)/#{f}")
+          if binding_location
+            b.eval(src, file, lineno)
+          else
+            b.eval(src, "(rdbg)/#{file}")
+          end
         end
       else
         frame_self = current_frame.self
@@ -411,16 +432,16 @@ module DEBUGGER__
       [:return_value,     "_return"],
     ]
 
-    def frame_eval src, re_raise: false
+    def frame_eval src, re_raise: false, binding_location: false
       @success_last_eval = false
 
-      b = current_frame.eval_binding
+      b = current_frame&.eval_binding || TOPLEVEL_BINDING
 
       special_local_variables current_frame do |name, var|
         b.local_variable_set(name, var) if /\%/ !~ name
       end
 
-      result = frame_eval_core(src, b)
+      result = frame_eval_core(src, b, binding_location: binding_location)
 
       @success_last_eval = true
       result
@@ -482,19 +503,28 @@ module DEBUGGER__
       exit!
     end
 
-    def show_src(frame_index: @current_frame_index, update_line: false, max_lines: CONFIG[:show_src_lines], **options)
+    def show_src(frame_index: @current_frame_index, update_line: false, ignore_show_line: false, max_lines: CONFIG[:show_src_lines], **options)
       if frame = get_frame(frame_index)
-        start_line, end_line, lines = *get_src(frame, max_lines: max_lines, **options)
-
-        if start_line
-          if update_line
-            frame.show_line = end_line
+        begin
+          if ignore_show_line
+            prev_show_line = frame.show_line
+            frame.show_line = nil
           end
 
-          puts "[#{start_line+1}, #{end_line}] in #{frame.pretty_path}" if !update_line && max_lines != 1
-          puts lines[start_line...end_line]
-        else
-          puts "# No sourcefile available for #{frame.path}"
+          start_line, end_line, lines = *get_src(frame, max_lines: max_lines, **options)
+
+          if start_line
+            if update_line
+              frame.show_line = end_line
+            end
+
+            puts "[#{start_line+1}, #{end_line}] in #{frame.pretty_path}" if !update_line && max_lines != 1
+            puts lines[start_line...end_line]
+          else
+            puts "# No sourcefile available for #{frame.path}"
+          end
+        ensure
+          frame.show_line = prev_show_line if prev_show_line
         end
       end
     end
@@ -546,27 +576,55 @@ module DEBUGGER__
       end
     end
 
-    def show_ivars pat
-      if s = current_frame&.self
-        M_INSTANCE_VARIABLES.bind_call(s).sort.each{|iv|
-          value = M_INSTANCE_VARIABLE_GET.bind_call(s, iv)
+    def show_ivars pat, expr = nil
+      if expr && !expr.empty?
+        _self = frame_eval(expr);
+      elsif _self = current_frame&.self
+      else
+        _self = nil
+      end
+
+      if _self
+        M_INSTANCE_VARIABLES.bind_call(_self).sort.each{|iv|
+          value = M_INSTANCE_VARIABLE_GET.bind_call(_self, iv)
           puts_variable_info iv, value, pat
         }
       end
     end
 
-    def show_consts pat, only_self: false
-      if s = current_frame&.self
-        cs = {}
-        if M_KIND_OF_P.bind_call(s, Module)
-          cs[s] = :self
+    def iter_consts c, names = {}
+      c.constants(false).sort.each{|name|
+        next if names.has_key? name
+        names[name] = nil
+        begin
+          value = c.const_get(name)
+        rescue Exception => e
+          value = e
+        end
+        yield name, value
+      }
+    end
+
+    def get_consts expr = nil, only_self: false, &block
+      if expr && !expr.empty?
+        _self = frame_eval(expr)
+        if M_KIND_OF_P.bind_call(_self, Module)
+          iter_consts _self, &block
+          return
         else
-          s = M_CLASS.bind_call(s)
-          cs[s] = :self unless only_self
+          raise "#{_self.inspect} (by #{expr}) is not a Module."
+        end
+      elsif _self = current_frame&.self
+        cs = {}
+        if M_KIND_OF_P.bind_call(_self, Module)
+          cs[_self] = :self
+        else
+          _self = M_CLASS.bind_call(_self)
+          cs[_self] = :self unless only_self
         end
 
         unless only_self
-          s.ancestors.each{|c| break if c == Object; cs[c] = :ancestors}
+          _self.ancestors.each{|c| break if c == Object; cs[c] = :ancestors}
           if b = current_frame&.binding
             b.eval('::Module.nesting').each{|c| cs[c] = :nesting unless cs.has_key? c}
           end
@@ -575,19 +633,19 @@ module DEBUGGER__
         names = {}
 
         cs.each{|c, _|
-          c.constants(false).sort.each{|name|
-            next if names.has_key? name
-            names[name] = nil
-            value = c.const_get(name)
-            puts_variable_info name, value, pat
-          }
+          iter_consts c, names, &block
         }
       end
     end
 
-    SKIP_GLOBAL_LIST = %i[$= $KCODE $-K $SAFE].freeze
+    def show_consts pat, expr = nil, only_self: false
+      get_consts expr, only_self: only_self do |name, value|
+        puts_variable_info name, value, pat
+      end
+    end
+
     def show_globals pat
-      global_variables.sort.each{|name|
+      safe_global_variables.sort.each{|name|
         next if SKIP_GLOBAL_LIST.include? name
 
         value = eval(name.to_s)
@@ -643,7 +701,8 @@ module DEBUGGER__
         if editor = (ENV['RUBY_DEBUG_EDITOR'] || ENV['EDITOR'])
           puts "command: #{editor}"
           puts "   path: #{path}"
-          system(editor, path)
+          require 'shellwords'
+          system(*Shellwords.split(editor), path)
         else
           puts "can not find editor setting: ENV['RUBY_DEBUG_EDITOR'] or ENV['EDITOR']"
         end
@@ -758,7 +817,7 @@ module DEBUGGER__
       case args.first
       when :method
         klass_name, op, method_name, cond, cmd, path = args[1..]
-        bp = MethodBreakpoint.new(current_frame.eval_binding, klass_name, op, method_name, cond: cond, command: cmd, path: path)
+        bp = MethodBreakpoint.new(current_frame&.eval_binding || TOPLEVEL_BINDING, klass_name, op, method_name, cond: cond, command: cmd, path: path)
         begin
           bp.enable
         rescue NameError => e
@@ -814,6 +873,7 @@ module DEBUGGER__
           set_mode :waiting if !waiting?
           cmds = @q_cmd.pop
           # pp [self, cmds: cmds]
+
           break unless cmds
         ensure
           set_mode :running
@@ -831,8 +891,9 @@ module DEBUGGER__
 
           case step_type
           when :in
+            iter = iter || 1
             if @recorder&.replaying?
-              @recorder.step_forward
+              @recorder.step_forward iter
               raise SuspendReplay
             else
               step_tp iter do
@@ -845,6 +906,7 @@ module DEBUGGER__
             frame = @target_frames.first
             path = frame.location.absolute_path || "!eval:#{frame.path}"
             line = frame.location.lineno
+            label = frame.location.base_label
 
             if frame.iseq
               frame.iseq.traceable_lines_norec(lines = {})
@@ -856,35 +918,89 @@ module DEBUGGER__
 
             depth = @target_frames.first.frame_depth
 
-            step_tp iter do
+            step_tp iter do |tp|
               loc = caller_locations(2, 1).first
               loc_path = loc.absolute_path || "!eval:#{loc.path}"
+              loc_label = loc.base_label
+              loc_depth = DEBUGGER__.frame_depth - 3
 
-              # same stack depth
-              (DEBUGGER__.frame_depth - 3 <= depth) ||
-
-              # different frame
-              (next_line && loc_path == path &&
-               (loc_lineno = loc.lineno) > line &&
-               loc_lineno <= next_line)
+              case
+              when loc_depth == depth && loc_label == label
+                true
+              when loc_depth < depth
+                # lower stack depth
+                true
+              when (next_line &&
+                    loc_path == path &&
+                    (loc_lineno = loc.lineno) > line &&
+                    loc_lineno <= next_line)
+                # different frame (maybe block) but the line is before next_line
+                true
+              end
             end
             break
 
           when :finish
             finish_frames = (iter || 1) - 1
-            goal_depth = @target_frames.first.frame_depth - finish_frames
+            frame = @target_frames.first
+            goal_depth = frame.frame_depth - finish_frames - (frame.has_return_value ? 1 : 0)
 
             step_tp nil, [:return, :b_return] do
               DEBUGGER__.frame_depth - 3 <= goal_depth ? true : false
             end
             break
 
+          when :until
+            location = iter&.strip
+            frame = @target_frames.first
+            depth = frame.frame_depth - (frame.has_return_value ? 1 : 0)
+            target_location_label = frame.location.base_label
+
+            case location
+            when nil, /\A(?:(.+):)?(\d+)\z/
+              no_loc = !location
+              file = $1 || frame.location.path
+              line = ($2 ||  frame.location.lineno + 1).to_i
+
+              step_tp nil, [:line, :return] do |tp|
+                if tp.event == :line
+                  next false if no_loc && depth < DEBUGGER__.frame_depth - 3
+                  next false unless tp.path.end_with?(file)
+                  next false unless tp.lineno >= line
+                  true
+                else
+                  true if depth >= DEBUGGER__.frame_depth - 3 &&
+                          caller_locations(2, 1).first.label == target_location_label
+                          # TODO: imcomplete condition
+                end
+              end
+            else
+              pat = location
+              if /\A\/(.+)\/\z/ =~ pat
+                pat = Regexp.new($1)
+              end
+
+              step_tp nil, [:call, :c_call, :return] do |tp|
+                case tp.event
+                when :call, :c_call
+                  true if pat === tp.callee_id.to_s
+                else # :return, :b_return
+                  true if depth >= DEBUGGER__.frame_depth - 3 &&
+                          caller_locations(2, 1).first.label == target_location_label
+                          # TODO: imcomplete condition
+                end
+              end
+            end
+
+            break
+
           when :back
+            iter = iter || 1
             if @recorder&.can_step_back?
               unless @recorder.backup_frames
                 @recorder.backup_frames = @target_frames
               end
-              @recorder.step_back
+              @recorder.step_back iter
               raise SuspendReplay
             else
               puts "Can not step back more."
@@ -922,8 +1038,9 @@ module DEBUGGER__
           when :call
             result = frame_eval(eval_src)
           when :irb
+            require 'irb' # prelude's binding.irb doesn't have show_code option
             begin
-              result = frame_eval('binding.irb')
+              result = frame_eval('binding.irb(show_code: false)', binding_location: true)
             ensure
               # workaround: https://github.com/ruby/debug/issues/308
               Reline.prompt_proc = nil if defined? Reline
@@ -988,6 +1105,10 @@ module DEBUGGER__
           when :list
             show_src(update_line: true, **(args.first || {}))
 
+          when :whereami
+            show_src ignore_show_line: true
+            show_frames CONFIG[:show_frames]
+
           when :edit
             show_by_editor(args.first)
 
@@ -1003,11 +1124,13 @@ module DEBUGGER__
 
           when :ivars
             pat = args.shift
-            show_ivars pat
+            expr = args.shift
+            show_ivars pat, expr
 
           when :consts
             pat = args.shift
-            show_consts pat
+            expr = args.shift
+            show_consts pat, expr
 
           when :globals
             pat = args.shift
@@ -1093,6 +1216,8 @@ module DEBUGGER__
           end
           event! :result, nil
 
+        when :quit
+          sleep # wait for SystemExit
         when :dap
           process_dap args
         when :cdp
@@ -1107,6 +1232,8 @@ module DEBUGGER__
     rescue Exception => e
       pp ["DEBUGGER Exception: #{__FILE__}:#{__LINE__}", e, e.backtrace]
       raise
+    ensure
+      @returning = false
     end
 
     def debug_event(ev, args)
@@ -1187,12 +1314,18 @@ module DEBUGGER__
         @tp_recorder.enabled?
       end
 
-      def step_back
-        @index += 1
+      def step_back iter
+        @index += iter
+        if @index > @log.size
+          @index = @log.size
+        end
       end
 
-      def step_forward
-        @index -= 1
+      def step_forward iter
+        @index -= iter
+        if @index < 0
+          @index = 0
+        end
       end
 
       def step_reset
