@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module RBS
   class EnvironmentLoader
     class UnknownLibraryError < StandardError
@@ -21,7 +23,9 @@ module RBS
     DEFAULT_CORE_ROOT = Pathname(_ = __dir__) + "../../core"
 
     def self.gem_sig_path(name, version)
-      spec = Gem::Specification.find_by_name(name, version)
+      requirements = []
+      requirements << version if version
+      spec = Gem::Specification.find_by_name(name, *requirements)
       path = Pathname(spec.gem_dir) + "sig"
       if path.directory?
         [spec, path]
@@ -34,16 +38,37 @@ module RBS
       @core_root = core_root
       @repository = repository
 
-      @libs = []
+      @libs = Set.new
       @dirs = []
     end
 
-    def add(path: nil, library: nil, version: nil)
+    def add(path: nil, library: nil, version: nil, resolve_dependencies: true)
       case
       when path
         dirs << path
       when library
-        libs << Library.new(name: library, version: version)
+        if library == 'rubygems'
+          RBS.logger.warn '`rubygems` has been moved to core library, so it is always loaded. Remove explicit loading `rubygems`'
+          return
+        end
+
+        if libs.add?(Library.new(name: library, version: version)) && resolve_dependencies
+          resolve_dependencies(library: library, version: version)
+        end
+      end
+    end
+
+    def resolve_dependencies(library:, version:)
+      [Collection::Sources::Rubygems.instance, Collection::Sources::Stdlib.instance].each do |source|
+        # @type var gem: { 'name' => String, 'version' => String? }
+        gem = { 'name' => library, 'version' => version }
+        next unless source.has?(gem)
+
+        gem['version'] ||= source.versions(gem).last
+        source.dependencies_of(gem)&.each do |dep|
+          add(library: dep['name'], version: nil)
+        end
+        return
       end
     end
 
@@ -53,7 +78,7 @@ module RBS
       repository.add(collection_config.repo_path)
 
       collection_config.gems.each do |gem|
-        add(library: gem['name'], version: gem['version'])
+        add(library: gem['name'], version: gem['version'], resolve_dependencies: false)
       end
     end
 
