@@ -1,6 +1,10 @@
+# frozen_string_literal: true
+
 module RBS
   module Prototype
     class Runtime
+      include Helpers
+
       attr_reader :patterns
       attr_reader :env
       attr_reader :merge
@@ -19,6 +23,7 @@ module RBS
 
       def target?(const)
         name = const_name(const)
+        return false unless name
 
         patterns.any? do |pattern|
           if pattern.end_with?("*")
@@ -75,7 +80,7 @@ module RBS
           supers.merge(mix.included_modules)
         end
 
-        if mod.is_a?(Class)
+        if mod.is_a?(Class) && mod.superclass
           mod.superclass.included_modules.each do |mix|
             supers << mix
             supers.merge(mix.included_modules)
@@ -133,11 +138,14 @@ module RBS
           when :block
             block = Types::Block.new(
               type: Types::Function.empty(untyped).update(rest_positionals: Types::Function::Param.new(name: nil, type: untyped)),
-              required: true
+              required: true,
+              self_type: nil
             )
           end
         end
 
+        block ||= block_from_ast_of(method)
+
         return_type = if method.name == :initialize
                         Types::Bases::Void.new(location: nil)
                       else
@@ -314,7 +322,12 @@ module RBS
 
       def generate_constants(mod, decls)
         mod.constants(false).sort.each do |name|
-          value = mod.const_get(name)
+          begin
+            value = mod.const_get(name)
+          rescue StandardError, LoadError => e
+            RBS.logger.warn("Skipping constant #{name} of #{mod} since #{e}")
+            next
+          end
 
           next if value.is_a?(Class) || value.is_a?(Module)
           unless value.class.name
@@ -337,7 +350,7 @@ module RBS
                  end
 
           decls << AST::Declarations::Constant.new(
-            name: name,
+            name: to_type_name(name.to_s),
             type: type,
             location: nil,
             comment: nil
@@ -346,7 +359,7 @@ module RBS
       end
 
       def generate_super_class(mod)
-        if mod.superclass == ::Object
+        if mod.superclass.nil? || mod.superclass == ::Object
           nil
         elsif const_name(mod.superclass).nil?
           RBS.logger.warn("Skipping anonymous superclass #{mod.superclass} of #{mod}")
@@ -367,7 +380,7 @@ module RBS
         unless decl
           decl = AST::Declarations::Class.new(
             name: to_type_name(only_name(mod)),
-            type_params: AST::Declarations::ModuleTypeParams.empty,
+            type_params: [],
             super_class: generate_super_class(mod),
             members: [],
             annotations: [],
@@ -421,7 +434,7 @@ module RBS
         unless decl
           decl = AST::Declarations::Module.new(
             name: to_type_name(only_name(mod)),
-            type_params: AST::Declarations::ModuleTypeParams.empty,
+            type_params: [],
             self_types: [],
             members: [],
             annotations: [],
@@ -466,8 +479,9 @@ module RBS
         *outer_module_names, _ = const_name(mod).split(/::/) #=> parent = [A, B], mod = C
         destination = @decls # Copy the entries in ivar @decls, not .dup
 
-        outer_module_names&.each do |outer_module_name|
-          outer_module = @modules.detect { |x| const_name(x) == outer_module_name }
+        outer_module_names&.each_with_index do |outer_module_name, i|
+          current_name = outer_module_names[0, i + 1].join('::')
+          outer_module = @modules.detect { |x| const_name(x) == current_name }
           outer_decl = destination.detect { |decl| decl.is_a?(outer_module.is_a?(Class) ? AST::Declarations::Class : AST::Declarations::Module) && decl.name.name == outer_module_name.to_sym }
 
           # Insert AST::Declarations if declarations are not added previously
@@ -475,7 +489,7 @@ module RBS
             if outer_module.is_a?(Class)
               outer_decl = AST::Declarations::Class.new(
                 name: to_type_name(outer_module_name),
-                type_params: AST::Declarations::ModuleTypeParams.empty,
+                type_params: [],
                 super_class: generate_super_class(outer_module),
                 members: [],
                 annotations: [],
@@ -485,7 +499,7 @@ module RBS
             else
               outer_decl = AST::Declarations::Module.new(
                 name: to_type_name(outer_module_name),
-                type_params: AST::Declarations::ModuleTypeParams.empty,
+                type_params: [],
                 self_types: [],
                 members: [],
                 annotations: [],
@@ -522,6 +536,18 @@ module RBS
           []
         end
       end
+
+      def block_from_ast_of(method)
+        return nil if RUBY_VERSION < '3.1'
+
+        begin
+          ast = RubyVM::AbstractSyntaxTree.of(method)
+        rescue ArgumentError
+          return # When the method is defined in eval
+        end
+
+        block_from_body(ast) if ast&.type == :SCOPE
+      end
     end
   end
 end
